// file: timer.ino
//
// Po stisknuti tlacitka generuje impuls delky {10ms..~10s} 
// podle natoceni potenciometru. One shot timer.
//
// otestovano: n/*y @ arduino mega2560
// IDE 1.0.2
//
// A0; analog 0; potenciometr
// 12; DI 12; tlacitko zapojene ke gnd, externi pull-up R=10k
// 13; DO 13; vystup casovace '0'-klid, '1'-casovac bezi
// 
// - - - - - - - - - - - - - - -
#define DEBUG (0)

#if (DEBUG==1)
  #include <Dbg.h>
#endif

#define BREAD(_byte,_bitmask) ((_byte)&(_bitmask))

// - - - - - - - - - - - - - - -
const int sensorPin = A0;
const int buttonPin = 12;     // the number of the pushbutton pin
const int ledPin =  13;      // the number of the LED pin

// - - - - - - - - - - - - - - -
void set_output_hi( void);
void set_output_low( void);
void ton1( void);
void get_ad( void);
void get_switch( void);
void get_time( void);

// - - - - - - - - - - - - - - -
uint8_t temp;

typedef struct
{
  uint8_t button;
  uint8_t button_ip;
  uint8_t button_hf;
  uint8_t ton1_sb;
  uint16_t ton1_value;
  uint16_t ad0;

  uint8_t time;
  uint8_t time_ip;
  uint8_t time_hf;
  uint32_t millis_;
  uint32_t millis_old;
} t_global;

t_global g;


// - - - - - - - - - - - - - - -
void setup()
{
  pinMode(13, OUTPUT); // led
  digitalWrite(ledPin, LOW); // led

#if (DEBUG==1)
  Debug.begin(1,9600);
#endif
}


// - - - - - - - - - - - - - - -
void loop()
{
  get_time();
  get_switch();
  get_ad();

#if (DEBUG==1)
  dbg("%d", g.ad0);
#endif

  // start ton1 timer
  if (BREAD(g.button_ip,0x01))
  {
    if (g.ton1_sb!=0x02) // not run
    {
      g.ton1_value=g.ad0 + (+1); // +1 avoid zero time
      g.ton1_sb=0x02;
    }
  }
  ton1();
  if (g.ton1_sb==0x02)
  {
    set_output_hi();
  }
  else
  {
    set_output_low();
  }
}

// - - - - - - - - - - - - - - -
void set_output_hi( void)
{
  digitalWrite(ledPin, HIGH);
}

// - - - - - - - - - - - - - - -
void set_output_low( void)
{
  digitalWrite(ledPin, LOW);
}

// - - - - - - - - - - - - - - -
void ton1( void)
{
  if (BREAD(g.time_ip,0x01))
    if (g.ton1_sb==0x02)
    {
      if (0==g.ton1_value)
      {
        g.ton1_sb=0x04;
      }
      else
      {
        g.ton1_value--;
      }
    }
}

// - - - - - - - - - - - - - - -
void get_ad( void)
{
  g.ad0=(analogRead(sensorPin)); // full 10bit
}

// - - - - - - - - - - - - - - -
void get_switch( void)
{
  temp=(0==digitalRead(buttonPin))?1:0; // button pressed->hw:0V->sw:'1'
  g.button_ip = (temp) & (~g.button_hf);
  g.button_hf = temp;
}
// - - - - - - - - - - - - - - -
void get_time( void)
{
  g.millis_ = millis();
  if (5<=(g.millis_ - g.millis_old))
  {
    g.millis_old = g.millis_;
    g.time++; // every 5ms
  }

  {
    temp = g.time;
    g.time_ip = (~temp) & g.time_hf; // nail impuls every {...,10ms}
    g.time_hf = temp;
  }
}
// EOF

